@@ -190,17 +190,30 @@ module Mono = struct
       buf.(i) <- Sample.clip buf.(i)
     done
 
-  let resample ratio inbuf offs len =
+  let resample ?(mode=`Nearest) ratio inbuf offs len =
     if ratio = 1. then
       let outbuf = create len in
       blit inbuf offs outbuf 0 len;
       outbuf
+    else if mode = `Nearest then
+      let outlen = int_of_float (float len *. ratio) in
+      let outbuf = create outlen in
+      for i = 0 to outlen - 1 do
+        let pos = min (int_of_float ((float i /. ratio) +. 0.5)) (len - 1) in
+	outbuf.(i) <- inbuf.(pos + offs)
+      done;
+      outbuf
     else
       let outlen = int_of_float (float len *. ratio) in
       let outbuf = create outlen in
       for i = 0 to outlen - 1 do
-	let inidx = min (int_of_float (float i /. ratio)) (len - 1) in
-	outbuf.(i) <- inbuf.(inidx + offs)
+        let ir = float i /. ratio in
+	let pos = min (int_of_float ir) (len - 1) in
+        if pos = len - 1 then
+	  outbuf.(i) <- inbuf.(pos + offs)
+        else
+          let a = ir -. float pos in
+          outbuf.(i) <- inbuf.(pos + offs) *. (1. -. a) +. inbuf.(pos + offs) *. a
       done;
       outbuf
 
@@ -915,24 +928,18 @@ let to_mono b =
 
 let of_mono b = [|b|]
 
-let resample ratio buf ofs len =
-  map (fun buf -> Mono.resample ratio buf ofs len) buf
+let resample ?mode ratio buf ofs len =
+  map (fun buf -> Mono.resample ?mode ratio buf ofs len) buf
 
 module U8 = struct
-  external resample_to_audio :
-    string -> int -> int -> float -> buffer -> int -> int
-    = "caml_float_pcm_of_u8_resample_byte"
-      "caml_float_pcm_of_u8_resample_native"
+  external to_audio :
+    string -> int -> buffer -> int -> int -> unit
+    = "caml_float_pcm_of_u8_byte"
+      "caml_float_pcm_of_u8_native"
 
   external of_audio :
     float array array -> int -> string -> int -> int -> unit
     = "caml_float_pcm_to_u8"
-
-  let convert_to_audio s sofs slen ?(resample=1.) buf bofs =
-    resample_to_audio s sofs slen resample buf bofs
-
-  let to_audio s sofs buf bofs len =
-    ignore (resample_to_audio s sofs len 1. buf bofs)
 end
 
 module S16LE = struct
@@ -950,13 +957,7 @@ module S16LE = struct
     of_audio buf ofs sbuf 0 len;
     sbuf
 
-  external resample_to_audio : string -> int -> int -> float -> float array array -> int -> int = "caml_float_pcm_convert_s16le_byte" "caml_float_pcm_convert_s16le_native"
-
-  let convert_to_audio s sofs slen ?(resample=1.) buf bofs =
-    resample_to_audio s sofs slen resample buf bofs
-
-  (* TODO: unify the parameters.. *)
-  let to_audio s sofs buf bofs len = ignore(resample_to_audio s sofs len 1. buf bofs)
+  external to_audio : string -> int -> float array array -> int -> int -> unit = "caml_float_pcm_convert_s16le_byte" "caml_float_pcm_convert_s16le_native"
 end
 
 let add b1 o1 b2 o2 len = iter2 (fun b1 b2 -> Mono.add b1 o1 b2 o2 len) b1 b2
