@@ -84,7 +84,7 @@
 
 #ifdef HAVE_MEMALIGN
 /* some systems have memalign() but no declaration for it */
-void * memalign (size_t align, size_t size);
+void * memalign(size_t align, size_t size);
 #else
 /* assume malloc alignment is sufficient */
 #define memalign(align,size) malloc (size)
@@ -94,9 +94,9 @@ void * memalign (size_t align, size_t size);
 #define ALIGN(a) a=((a+ALIGNMENT_BYTES-1)/ALIGNMENT_BYTES)*ALIGNMENT_BYTES
 
 /* This function creates a 16 bytes 
- * alligned plane. It returns a big array 
+ * aligned plane. It returns a big array 
  * along with the new stride. */
-CAMLprim value caml_rgb_alligned_plane(value _height, value _stride)
+CAMLprim value caml_rgb_aligned_plane(value _height, value _stride)
 {
   CAMLparam0();
   CAMLlocal2(v,ans);
@@ -108,12 +108,11 @@ CAMLprim value caml_rgb_alligned_plane(value _height, value _stride)
   long len = height*stride;  
 
   // Init plane..
-  void *data = memalign(ALIGNMENT_BYTES,len);
-  if (data == NULL) 
-    caml_raise_out_of_memory();
+  void *data;
+  data = memalign(ALIGNMENT_BYTES,len);
+  if (data == NULL) caml_raise_out_of_memory();
   v = caml_ba_alloc(CAML_BA_MANAGED|CAML_BA_C_LAYOUT|CAML_BA_UINT8,1,data,&len);
 
-
   // We return
   ans = caml_alloc_tuple(2);
   Store_field(ans,0,Val_int(stride));
@@ -161,7 +160,8 @@ static frame *rgb_copy(frame *src, frame *dst)
   dst->width = src->width;
   dst->height = src->height;
   dst->stride = src->stride;
-  dst->data = malloc(Rgb_data_size(src));
+  dst->data = memalign(ALIGNMENT_BYTES, Rgb_data_size(src));
+  if (dst->data == NULL) caml_raise_out_of_memory();
   memcpy(dst->data, src->data, Rgb_data_size(src));
 
   return dst;
@@ -538,7 +538,8 @@ CAMLprim value caml_rgb_of_rgb8_string(value _rgb, value _data)
   frame rgb;
   frame_of_value(_rgb, &rgb);
   int datalen = rgb.height * rgb.width * 3;
-  char *data = (char*)malloc(datalen);
+  char *data = (char*)memalign(ALIGNMENT_BYTES, datalen);
+  if (data == NULL) caml_raise_out_of_memory();
 
   int i, j;
 
@@ -558,6 +559,28 @@ CAMLprim value caml_rgb_of_rgb8_string(value _rgb, value _data)
   CAMLreturn(Val_unit);
 }
 
+CAMLprim value caml_rgba_of_bgra(value _rgba, value _bgra)
+{
+  CAMLparam2(_rgba, _bgra);
+  frame rgba, bgra;
+  frame_of_value(_rgba, &rgba);
+  frame_of_value(_bgra, &bgra);
+  int i,j;
+
+  caml_enter_blocking_section();
+  for (j = 0; j < bgra.height; j++)
+    for (i = 0; i < bgra.width; i++)
+      {
+        Red(&rgba,i,j) = Color(&bgra,2,i,j);
+        Green(&rgba,i,j) = Color(&bgra,1,i,j);
+        Blue(&rgba,i,j) = Color(&bgra,0,i,j);
+        Alpha(&rgba,i,j) = Color(&bgra,3,i,j);
+      }
+  caml_leave_blocking_section();
+
+  CAMLreturn(Val_unit);
+}
+
 CAMLprim value caml_rgb_get_pixel(value f, value _x, value _y)
 {
   CAMLparam1(f);
@@ -774,6 +797,7 @@ CAMLprim value caml_rgb_to_bmp(value _rgb)
   frame_of_value(_rgb,&rgb);
   int len = Rgb_num_pix(&rgb);
   char *bmp = malloc(54 + 3 * len);
+  if (bmp == NULL) caml_raise_out_of_memory();
   int i, j;
   unsigned char a;
 
@@ -822,6 +846,7 @@ CAMLprim value caml_image_to_rgb24(value _rgb)
   frame_of_value(_rgb,&rgb);
   int len = Rgb_num_pix(&rgb);
   char *bmp = malloc(3 * len);
+  if (bmp == NULL) caml_raise_out_of_memory();
   int i, j;
   unsigned char a;
 
@@ -929,6 +954,27 @@ CAMLprim value caml_rgb_greyscale(value _rgb, value _sepia)
   CAMLreturn(Val_unit);
 }
 
+CAMLprim value caml_rgba_swap_rb(value _rgba)
+{
+  CAMLparam1(_rgba);
+  frame rgba;
+  frame_of_value(_rgba, &rgba);
+  int i, j;
+  unsigned char c;
+
+  caml_enter_blocking_section();
+  for (j = 0; j < rgba.height; j++)
+    for (i = 0; i < rgba.width; i++)
+      {
+        c = Red(&rgba,i,j);
+        Red(&rgba,i,j) = Blue(&rgba,i,j);
+        Blue(&rgba,i,j) = c;
+      }
+  caml_leave_blocking_section();
+
+  CAMLreturn(Val_unit);
+}
+
 CAMLprim value caml_rgb_add(value _dst, value _src)
 {
   CAMLparam2(_src,_dst);
@@ -1173,24 +1219,22 @@ CAMLprim value caml_rgb_affine(value _rgb, value _ax, value _ay, value _ox, valu
   int i, j, i2, j2, c;
   int ox = Int_val(_ox),
       oy = Int_val(_oy);
-  int dx = rgb.width / 2,  /* Center of scaling */
-      dy = rgb.height / 2;
-  int istart = max(0, (ox - dx) * ax + dx),
-      iend = min(rgb.width, (rgb.width + ox + dx) * ax + dx),
-      jstart = max(0, (oy - dy) * ay + dy),
-      jend = min(rgb.height, (rgb.height + ox + dx) * ax + dx);
+  int istart = max(0, ox),
+      iend = min(rgb.width, rgb.width*ax+ox),
+      jstart = max(0, oy),
+      jend = min(rgb.height, rgb.height*ay+oy);
 
   caml_enter_blocking_section();
   rgb_blank(&rgb);
   for (j = jstart; j < jend; j++)
     for (i = istart; i < iend; i++)
     {
-      i2 = (i - dx) / ax + dx - ox;
-      j2 = (j - dy) / ay + dy - oy;
+      i2 = (i-ox)/ax;
+      j2 = (j-oy)/ay;
       /* TODO: this test shouldn't be needed */
-      if (!Is_outside(&old, i2, j2))
-        for (c = 0; c < Rgb_elems_per_pixel; c++)
-          Color(&rgb, c, i, j) = Color(&old, c, i2, j2);
+      /* if (!Is_outside(&old, i2, j2)) */
+      for (c = 0; c < Rgb_elems_per_pixel; c++)
+        Color(&rgb, c, i, j) = Color(&old, c, i2, j2);
     }
   caml_leave_blocking_section();
 
@@ -1521,6 +1565,7 @@ CAMLprim value caml_mm_Gray8_motion_multi_compute(value _bs, value _width, value
   intnat vlen = vw*vh*2;
   // Vector table of size vw*vh
   int *v = malloc(vlen*sizeof(int));
+  if (v == NULL) caml_raise_out_of_memory();
   // Current score
   int s00, s10, s01, s11;
   // Best score
@@ -1594,6 +1639,7 @@ CAMLprim value caml_rgb_motion_multi_median_denoise(value _vw, value _v)
 
   caml_enter_blocking_section();
   oldv = malloc(len * 2 * sizeof(int));
+  if (oldv == NULL) caml_raise_out_of_memory();
   memcpy(oldv, v, len * 2 * sizeof(int));
   for (j = 1; j < vh - 1; j++)
     for (i = 1; i < vw - 1; i++)
